@@ -4,16 +4,20 @@
 #include <facter/facts/scalar_value.hpp>
 #include <facter/facts/array_value.hpp>
 #include <facter/facts/map_value.hpp>
-#include <facter/util/directory.hpp>
-#include <facter/util/environment.hpp>
+#include <facter/ruby/ruby.hpp>
 #include <facter/util/string.hpp>
 #include <facter/version.h>
-#include <internal/util/dynamic_library.hpp>
+#include <internal/facts/resolvers/hypervisors_resolver.hpp>
 #include <internal/facts/resolvers/ruby_resolver.hpp>
 #include <internal/facts/resolvers/path_resolver.hpp>
 #include <internal/facts/resolvers/ec2_resolver.hpp>
 #include <internal/facts/resolvers/gce_resolver.hpp>
 #include <internal/facts/resolvers/augeas_resolver.hpp>
+#include <internal/ruby/ruby_value.hpp>
+#include <internal/facts/cache.hpp>
+#include <leatherman/dynamic_library/dynamic_library.hpp>
+#include <leatherman/file_util/directory.hpp>
+#include <leatherman/util/environment.hpp>
 #include <leatherman/logging/logging.hpp>
 #include <boost/filesystem.hpp>
 #include <boost/algorithm/string.hpp>
@@ -27,10 +31,14 @@ using namespace facter::util;
 using namespace rapidjson;
 using namespace YAML;
 using namespace boost::filesystem;
+using namespace leatherman::util;
+using namespace leatherman::file_util;
+using facter::util::maybe_stoi;
 
 namespace facter { namespace facts {
 
-    collection::collection()
+    collection::collection(set<string> const& blocklist, unordered_map<string, int64_t> const& ttls, bool ignore_cache) :
+        _blocklist(blocklist), _ttls(ttls), _ignore_cache(ignore_cache)
     {
         // This needs to be defined here since we use incomplete types in the header
     }
@@ -52,6 +60,8 @@ namespace facter { namespace facts {
             _resolvers = std::move(other._resolvers);
             _resolver_map = std::move(other._resolver_map);
             _pattern_resolvers = std::move(other._pattern_resolvers);
+            _blocklist = std::move(other._blocklist);
+            _ttls = std::move(other._ttls);
         }
         return *this;
     }
@@ -89,19 +99,23 @@ namespace facter { namespace facts {
                 ostringstream old_value_ss;
                 old_value->write(old_value_ss);
                 if (!value) {
-                    LOG_DEBUG("fact \"%1%\" resolved to null and the existing value of %2% will be removed.", name, old_value_ss.str());
+                    LOG_DEBUG("fact \"{1}\" resolved to null and the existing value of {2} will be removed.", name, old_value_ss.str());
                 } else {
                     ostringstream new_value_ss;
                     value->write(new_value_ss);
-                    LOG_DEBUG("fact \"%1%\" has changed from %2% to %3%.", name, old_value_ss.str(), new_value_ss.str());
+                    if (old_value->weight() > value->weight()) {
+                      LOG_DEBUG("new value for fact \"{1}\" ignored, because it's a lower weight", name);
+                    } else {
+                      LOG_DEBUG("fact \"{1}\" has changed from {2} to {3}.", name, old_value_ss.str(), new_value_ss.str());
+                    }
                 }
             } else {
                 if (!value) {
-                    LOG_DEBUG("fact \"%1%\" resolved to null and will not be added.", name);
+                    LOG_DEBUG("fact \"{1}\" resolved to null and will not be added.", name);
                 } else {
                     ostringstream new_value_ss;
                     value->write(new_value_ss);
-                    LOG_DEBUG("fact \"%1%\" has resolved to %2%.", name, new_value_ss.str());
+                    LOG_DEBUG("fact \"{1}\" has resolved to {2}.", name, new_value_ss.str());
                 }
             }
         }
@@ -113,30 +127,48 @@ namespace facter { namespace facts {
             return;
         }
 
+        // keep existing value if it has a larger weight value
+        if (old_value && old_value->weight() > value->weight())
+            return;
+
         _facts[move(name)] = move(value);
     }
 
+    void collection::add_custom(string name, unique_ptr<value> value, size_t weight)
+    {
+        if (value)
+            value->weight(weight);
+        add(move(name), move(value));
+    }
+
+    void collection::add_external(string name, unique_ptr<value> value)
+    {
+        if (value)
+            value->weight(external_fact_weight);
+        add(move(name), move(value));
+    }
+
     bool collection::add_external_facts_dir(vector<unique_ptr<external::resolver>> const& resolvers, string const& dir, bool warn)
     {
         // If dir is relative, make it an absolute path before passing to can_resolve.
         bool found = false;
         boost::system::error_code ec;
-        path search_dir = canonical(dir, ec);
+        path search_dir = absolute(dir);
 
-        if (ec || !is_directory(search_dir, ec)) {
+        if (!is_directory(search_dir, ec)) {
             // Warn the user if not using the default search directories
             string msg = ec ? ec.message() : "not a directory";
             if (warn) {
-                LOG_WARNING("skipping external facts for \"%1%\": %2%", dir, msg);
+                LOG_WARNING("skipping external facts for \"{1}\": {2}", dir, msg);
             } else {
-                LOG_DEBUG("skipping external facts for \"%1%\": %2%", dir, msg);
+                LOG_DEBUG("skipping external facts for \"{1}\": {2}", dir, msg);
             }
             return found;
         }
 
-        LOG_DEBUG("searching %1% for external facts.", search_dir);
+        LOG_DEBUG("searching {1} for external facts.", search_dir);
 
-        directory::each_file(search_dir.string(), [&](string const& path) {
+        each_file(search_dir.string(), [&](string const& path) {
             for (auto const& res : resolvers) {
                 if (res->can_resolve(path)) {
                     try {
@@ -144,7 +176,7 @@ namespace facter { namespace facts {
                         res->resolve(path, *this);
                     }
                     catch (external::external_fact_exception& ex) {
-                        LOG_ERROR("error while processing \"%1%\" for external facts: %2%", path, ex.what());
+                        LOG_ERROR("error while processing \"{1}\" for external facts: {2}", path, ex.what());
                     }
                     break;
                 }
@@ -184,10 +216,12 @@ namespace facter { namespace facts {
 
             auto fact_name = name.substr(7);
             boost::to_lower(fact_name);
-            LOG_DEBUG("setting fact \"%1%\" based on the value of environment variable \"%2%\".", fact_name, name);
+            LOG_DEBUG("setting fact \"{1}\" based on the value of environment variable \"{2}\".", fact_name, name);
 
             // Add the value based on the environment variable
-            add(fact_name, make_value<string_value>(move(value)));
+            auto fact_value = make_value<string_value>(move(value));
+            fact_value->weight(external_fact_weight);
+            add(fact_name, move(fact_value));
             if (callback) {
                 callback(fact_name);
             }
@@ -242,6 +276,24 @@ namespace facter { namespace facts {
         return _facts.empty() && _resolvers.empty();
     }
 
+    map<string, vector<string>> collection::get_fact_groups() {
+        map<string, vector<string>> fact_groups;
+        for (auto res : _resolvers) {
+            fact_groups.emplace(res->name(), res->names());
+        }
+        return fact_groups;
+    }
+
+    map<string, vector<string>> collection::get_blockable_fact_groups() {
+        map<string, vector<string>> blockgroups;
+        for (auto res : _resolvers) {
+            if (res->is_blockable()) {
+                blockgroups.emplace(res->name(), res->names());
+            }
+        }
+        return blockgroups;
+    }
+
     size_t collection::size()
     {
         resolve_facts();
@@ -264,10 +316,10 @@ namespace facter { namespace facts {
 
     ostream& collection::write(ostream& stream, format fmt, set<string> const& queries)
     {
-        return write(stream, fmt, queries, false);
+        return write(stream, fmt, queries, false, false);
     }
 
-    ostream& collection::write(ostream& stream, format fmt, set<string> const& queries, bool show_legacy)
+    ostream& collection::write(ostream& stream, format fmt, set<string> const& queries, bool show_legacy, bool strict_errors)
     {
         if (queries.empty()) {
             // Resolve all facts
@@ -275,24 +327,62 @@ namespace facter { namespace facts {
         }
 
         if (fmt == format::hash) {
-            write_hash(stream, queries, show_legacy);
+            write_hash(stream, queries, show_legacy, strict_errors);
         } else if (fmt == format::json) {
-            write_json(stream, queries, show_legacy);
+            write_json(stream, queries, show_legacy, strict_errors);
         } else if (fmt == format::yaml) {
-            write_yaml(stream, queries, show_legacy);
+            write_yaml(stream, queries, show_legacy, strict_errors);
         }
         return stream;
     }
 
+    bool collection::try_block(shared_ptr<resolver> const& res) {
+        if (_blocklist.count(res->name())) {
+            if (res->is_blockable()) {
+                LOG_DEBUG("blocking collection of {1} facts.", res->name());
+                return true;
+            } else {
+                LOG_DEBUG("{1} resolver cannot be blocked.", res->name());
+            }
+        }
+        return false;
+    }
+
+    void collection::resolve(shared_ptr<resolver> const& res) {
+        remove(res);
+
+        // Check if the resolver has been blocked
+        if (try_block(res)) {
+            return;
+        }
+
+        // Check if the resolver should be cached
+        auto resolver_ttl = _ttls.find(res->name());
+        if (!_ignore_cache && resolver_ttl != _ttls.end()) {
+           cache::use_cache(*this, res, (*resolver_ttl).second);
+           return;
+        }
+
+        // Resolve normally
+        LOG_DEBUG("resolving {1} facts.", res->name());
+        try {
+            res->resolve(*this);
+        } catch (std::runtime_error &e) {
+            LOG_WARNING("exception resolving {1} facts, some facts will not be available: {2}", res->name(), e.what());
+        }
+    }
+
     void collection::resolve_facts()
     {
+        // Delete any unused cache files
+        if (!_ignore_cache) {
+            cache::clean_cache(_ttls);
+        }
         // Remove the front of the resolvers list and resolve until no resolvers are left
         while (!_resolvers.empty()) {
             auto resolver = _resolvers.front();
-            remove(resolver);
-            LOG_DEBUG("resolving %1% facts.", resolver->name());
-            resolver->resolve(*this);
-        }
+            resolve(resolver);
+       }
     }
 
     void collection::resolve_fact(string const& name)
@@ -302,9 +392,7 @@ namespace facter { namespace facts {
         auto it = range.first;
         while (it != range.second) {
             auto resolver = (it++)->second;
-            remove(resolver);
-            LOG_DEBUG("resolving %1% facts.", resolver->name());
-            resolver->resolve(*this);
+            resolve(resolver);
         }
 
          // Resolve every resolver that matches the given name
@@ -315,9 +403,7 @@ namespace facter { namespace facts {
                 continue;
             }
             auto resolver = *(pattern_it++);
-            remove(resolver);
-            LOG_DEBUG("resolving %1% facts.", resolver->name());
-            resolver->resolve(*this);
+            resolve(resolver);
         }
     }
 
@@ -330,7 +416,7 @@ namespace facter { namespace facts {
         return it == _facts.end() ? nullptr : it->second.get();
     }
 
-    value const* collection::query_value(string const& query)
+    value const* collection::query_value(string const& query, bool strict_errors)
     {
         // First attempt to lookup a fact with the exact name of the query
         value const* current = get_value(query);
@@ -339,6 +425,7 @@ namespace facter { namespace facts {
         }
 
         bool in_quotes = false;
+        vector<string> segments;
         string segment;
         for (auto const& c : query) {
             if (c == '"') {
@@ -349,25 +436,46 @@ namespace facter { namespace facts {
                 segment += c;
                 continue;
             }
-            current = lookup(current, segment);
-            if (!current) {
-                return nullptr;
-            }
+            segments.emplace_back(move(segment));
             segment.clear();
         }
-
         if (!segment.empty()) {
-            current = lookup(current, segment);
+            segments.emplace_back(move(segment));
         }
+
+        auto segment_end = end(segments);
+        for (auto segment = begin(segments); segment != segment_end; ++segment) {
+            auto rb_val = dynamic_cast<ruby::ruby_value const *>(current);
+            if (rb_val) {
+                current = facter::ruby::lookup(current, segment, segment_end);
+                if (!current) {
+                    LOG_DEBUG("cannot lookup an element with \"{1}\" from Ruby fact", *segment);
+                }
+                // Once we hit Ruby there's no going back, so whatever we get from Ruby is the value.
+                return current;
+            } else {
+                current = lookup(current, *segment, strict_errors);
+            }
+            if (!current) {
+                // Break out early if there's no value for this segment
+                return nullptr;
+            }
+        }
+
         return current;
     }
 
-    value const* collection::lookup(value const* value, string const& name)
+    value const* collection::lookup(value const* value, string const& name, bool strict_errors)
     {
         if (!value) {
             value = get_value(name);
             if (!value) {
-                LOG_DEBUG("fact \"%1%\" does not exist.", name);
+                string message = "fact \"{1}\" does not exist.";
+                if (strict_errors) {
+                    LOG_ERROR(message, name);
+                } else {
+                    LOG_DEBUG(message, name);
+                }
             }
             return value;
         }
@@ -376,42 +484,46 @@ namespace facter { namespace facts {
         if (map) {
             value = (*map)[name];
             if (!value) {
-                LOG_DEBUG("cannot lookup a hash element with \"%1%\": element does not exist.", name);
+                LOG_DEBUG("cannot lookup a hash element with \"{1}\": element does not exist.", name);
             }
             return value;
         }
 
         auto array = dynamic_cast<array_value const*>(value);
-        if (array) {
-            int index;
-            try {
-                index = stoi(name);
-            } catch (logic_error&) {
-                LOG_DEBUG("cannot lookup an array element with \"%1%\": expected an integral value.", name);
-                return nullptr;
-            }
-            if (index < 0) {
-                LOG_DEBUG("cannot lookup an array element with \"%1%\": expected a non-negative value.", name);
-                return nullptr;
-            }
-            if (array->empty()) {
-                LOG_DEBUG("cannot lookup an array element with \"%1%\": the array is empty.", name);
-                return nullptr;
-            }
-            if (static_cast<size_t>(index) >= array->size()) {
-                LOG_DEBUG("cannot lookup an array element with \"%1%\": expected an integral value between 0 and %2% (inclusive).", name, array->size() - 1);
-                return nullptr;
-            }
-            return (*array)[index];
+        if (!array) {
+            return nullptr;
         }
-        return nullptr;
+
+        auto maybe_index = maybe_stoi(name);;
+        if (!maybe_index) {
+            LOG_DEBUG("cannot lookup an array element with \"{1}\": expected an integral value.", name);
+            return nullptr;
+        }
+
+        int index = maybe_index.get();
+        if (index < 0) {
+            LOG_DEBUG("cannot lookup an array element with \"{1}\": expected a non-negative value.", name);
+            return nullptr;
+        }
+
+        if (array->empty()) {
+            LOG_DEBUG("cannot lookup an array element with \"{1}\": the array is empty.", name);
+            return nullptr;
+        }
+
+        if (static_cast<size_t>(index) >= array->size()) {
+            LOG_DEBUG("cannot lookup an array element with \"{1}\": expected an integral value between 0 and {2} (inclusive).", name, array->size() - 1);
+            return nullptr;
+        }
+
+        return (*array)[index];
     }
 
-    void collection::write_hash(ostream& stream, set<string> const& queries, bool show_legacy)
+    void collection::write_hash(ostream& stream, set<string> const& queries, bool show_legacy, bool strict_errors)
     {
         // If there's only one query, print the result without the name
         if (queries.size() == 1u) {
-            auto value = query_value(*queries.begin());
+            auto value = query_value(*queries.begin(), strict_errors);
             if (value) {
                 value->write(stream, false);
             }
@@ -439,7 +551,7 @@ namespace facter { namespace facts {
             // Print queried facts
             vector<pair<string, value const*>> facts;
             for (auto const& query : queries) {
-                facts.push_back(make_pair(query, this->query_value(query)));
+                facts.push_back(make_pair(query, this->query_value(query, strict_errors)));
             }
 
             for (auto const& kvp : facts) {
@@ -453,24 +565,9 @@ namespace facter { namespace facts {
         }
     }
 
-    struct stream_adapter
+    void collection::write_json(ostream& stream, set<string> const& queries, bool show_legacy, bool strict_errors)
     {
-        explicit stream_adapter(ostream& stream) : _stream(stream)
-        {
-        }
-
-        void Put(char c)
-        {
-            _stream << c;
-        }
-
-     private:
-         ostream& _stream;
-    };
-
-    void collection::write_json(ostream& stream, set<string> const& queries, bool show_legacy)
-    {
-        Document document;
+        json_document document;
         document.SetObject();
 
         auto builder = ([&](string const& key, value const* val) {
@@ -478,18 +575,18 @@ namespace facter { namespace facts {
             if (!show_legacy && queries.empty() && val && val->hidden()) {
                 return;
             }
-            rapidjson::Value value;
+            json_value value;
             if (val) {
                 val->to_json(document.GetAllocator(), value);
             } else {
                 value.SetString("", 0);
             }
-            document.AddMember(key.c_str(), value, document.GetAllocator());
+            document.AddMember(StringRef(key.c_str(), key.size()), value, document.GetAllocator());
         });
 
         if (!queries.empty()) {
             for (auto const& query : queries) {
-                builder(query, this->query_value(query));
+                builder(query, this->query_value(query, strict_errors));
             }
         } else {
             for (auto const& kvp : _facts) {
@@ -503,7 +600,7 @@ namespace facter { namespace facts {
         document.Accept(writer);
     }
 
-    void collection::write_yaml(ostream& stream, set<string> const& queries, bool show_legacy)
+    void collection::write_yaml(ostream& stream, set<string> const& queries, bool show_legacy, bool strict_errors)
     {
         Emitter emitter(stream);
         emitter << BeginMap;
@@ -528,7 +625,7 @@ namespace facter { namespace facts {
         if (!queries.empty()) {
             vector<pair<string, value const*>> facts;
             for (auto const& query : queries) {
-                facts.push_back(make_pair(query, this->query_value(query)));
+                facts.push_back(make_pair(query, this->query_value(query, strict_errors)));
             }
 
             for (auto const& kvp : facts) {
@@ -545,6 +642,9 @@ namespace facter { namespace facts {
     void collection::add_common_facts(bool include_ruby_facts)
     {
         add("facterversion", make_value<string_value>(LIBFACTER_VERSION));
+#ifdef AIO_AGENT_VERSION
+        add("aio_agent_version", make_value<string_value>(AIO_AGENT_VERSION));
+#endif
 
         if (include_ruby_facts) {
             add(make_shared<resolvers::ruby_resolver>());
@@ -553,6 +653,9 @@ namespace facter { namespace facts {
         add(make_shared<resolvers::ec2_resolver>());
         add(make_shared<resolvers::gce_resolver>());
         add(make_shared<resolvers::augeas_resolver>());
+#ifdef USE_WHEREAMI
+        add(make_shared<resolvers::hypervisors_resolver>());
+#endif
     }
 
 }}  // namespace facter::facts
