@@ -1,11 +1,12 @@
 #include <internal/facts/osx/operating_system_resolver.hpp>
-#include <facter/execution/execution.hpp>
+#include <leatherman/execution/execution.hpp>
 #include <boost/algorithm/string.hpp>
+#include <boost/regex.hpp>
 #include <string>
 
 using namespace std;
 using namespace facter::facts;
-using namespace facter::execution;
+using namespace leatherman::execution;
 
 namespace facter { namespace facts { namespace osx {
 
@@ -14,7 +15,7 @@ namespace facter { namespace facts { namespace osx {
         // Default to the base implementation
         data result = posix::operating_system_resolver::collect_data(facts);
 
-        execution::each_line("/usr/bin/sw_vers", [&](string& line) {
+        each_line("/usr/bin/sw_vers", [&](string& line) {
             // Split at the first ':'
             auto pos = line.find(':');
             if (pos == string::npos) {
@@ -37,6 +38,11 @@ namespace facter { namespace facts { namespace osx {
             return result.osx.product.empty() || result.osx.build.empty() || result.osx.version.empty();
         });
 
+        // If osx.build is missing the patch version, add '.0'
+        if (boost::regex_match(result.osx.version, boost::regex("^\\d+\\.\\d+$"))) {
+            result.osx.version += ".0";
+        }
+
         return result;
     }
 
