@@ -1,10 +1,10 @@
 #include <internal/facts/windows/memory_resolver.hpp>
-#include <internal/util/windows/system_error.hpp>
-#include <internal/util/windows/windows.hpp>
+#include <leatherman/windows/system_error.hpp>
+#include <leatherman/windows/windows.hpp>
 #include <leatherman/logging/logging.hpp>
 #include <psapi.h>
 
-using namespace facter::util::windows;
+using namespace leatherman::windows;
 
 namespace facter { namespace facts { namespace windows {
 
@@ -12,7 +12,7 @@ namespace facter { namespace facts { namespace windows {
     {
         PERFORMANCE_INFORMATION statex;
         if (!GetPerformanceInfo(&statex, sizeof(statex))) {
-            LOG_DEBUG("resolving memory facts failed: %1%", system_error());
+            LOG_DEBUG("resolving memory facts failed: {1}", system_error());
             return {};
         }
 
