@@ -16,11 +16,11 @@ SCENARIO("using a double fact value") {
     REQUIRE(value.value() == Approx(42.4242));
     WHEN("serialized to JSON") {
         THEN("it should have the same value") {
-            rapidjson::Value json_value;
-            MemoryPoolAllocator<> allocator;
-            value.to_json(allocator, json_value);
-            REQUIRE(json_value.IsNumber());
-            REQUIRE(json_value.GetDouble() == Approx(42.4242));
+            json_value json;
+            json_allocator allocator;
+            value.to_json(allocator, json);
+            REQUIRE(json.IsNumber());
+            REQUIRE(json.GetDouble() == Approx(42.4242));
         }
     }
     WHEN("serialized to YAML") {
