@@ -6,7 +6,7 @@
 #include <facter/facts/array_value.hpp>
 #include <facter/facts/map_value.hpp>
 #include <facter/facts/scalar_value.hpp>
-#include <internal/util/regex.hpp>
+#include <leatherman/util/regex.hpp>
 #include <yaml-cpp/yaml.h>
 #include <boost/nowide/fstream.hpp>
 #include "../fixtures.hpp"
@@ -17,9 +17,12 @@
 #include <internal/facts/resolvers/dmi_resolver.hpp>
 #include <internal/facts/resolvers/ec2_resolver.hpp>
 #include <internal/facts/resolvers/filesystem_resolver.hpp>
+#include <internal/facts/resolvers/fips_resolver.hpp>
 #include <internal/facts/resolvers/gce_resolver.hpp>
+#include <internal/facts/resolvers/hypervisors_resolver.hpp>
 #include <internal/facts/resolvers/identity_resolver.hpp>
 #include <internal/facts/resolvers/kernel_resolver.hpp>
+#include <internal/facts/resolvers/ldom_resolver.hpp>
 #include <internal/facts/resolvers/load_average_resolver.hpp>
 #include <internal/facts/resolvers/memory_resolver.hpp>
 #include <internal/facts/resolvers/networking_resolver.hpp>
@@ -39,8 +42,9 @@
 
 using namespace std;
 using namespace facter::facts;
-using namespace facter::util;
+using namespace leatherman::util;
 using namespace facter::testing;
+using Catch::Matchers::AnyOf;
 
 // For every base resolver, implement a resolver that outputs the minimum values to pass schema validation
 // We don't care about the actual data in the facts, only that it conforms to the schema
@@ -94,6 +98,15 @@ struct dmi_resolver : resolvers::dmi_resolver
     }
 };
 
+struct ec2_resolver : resolvers::ec2_resolver
+{
+    void resolve(collection& facts) override
+    {
+        facts.add(fact::ec2_metadata, make_value<map_value>());
+        facts.add(fact::ec2_userdata, make_value<string_value>("user data"));
+    }
+};
+
 struct filesystem_resolver : resolvers::filesystem_resolver
 {
  protected:
@@ -121,11 +134,43 @@ struct filesystem_resolver : resolvers::filesystem_resolver
         p.label = "label";
         p.partition_label = "partlabel";
         p.mount = "mount";
+        p.backing_file = "/foo/bar";
         result.partitions.emplace_back(move(p));
         return result;
     }
 };
 
+struct fips_resolver : resolvers::fips_resolver
+{
+ protected:
+    virtual data collect_data(collection& facts) override
+    {
+        data result;
+        result.is_fips_mode_enabled = false;
+        return result;
+    }
+};
+
+using hypervisor_data = std::unordered_map<std::string, std::unordered_map<std::string, boost::variant<std::string, bool, int>>>;
+
+struct hypervisors_resolver_test : resolvers::hypervisors_resolver_base
+{
+    virtual hypervisor_data collect_data(collection& facts)
+    {
+        hypervisor_data results;
+
+        unordered_map<std::string, boost::variant<std::string, bool, int>> metadata {
+            {"string_value", string{"string"}}, // boost assumes const char* values are bools when bool is among the variants
+            {"integer_value", 42},
+            {"boolean_value", true},
+        };
+
+        results.insert({"hypervisor_name", metadata});
+
+        return results;
+    }
+};
+
 struct identity_resolver : resolvers::identity_resolver
 {
  protected:
@@ -136,6 +181,7 @@ struct identity_resolver : resolvers::identity_resolver
         result.group_name = "group";
         result.user_id = 456;
         result.user_name = "user";
+        result.privileged = false;
         return result;
     }
 };
@@ -192,12 +238,10 @@ struct networking_resolver : resolvers::networking_resolver
         interface iface;
         iface.name = "interface1";
         iface.dhcp_server = "192.168.1.1";
-        iface.address.v4 = "127.0.0.1";
-        iface.address.v6 = "fe80::1";
-        iface.netmask.v4 = "255.0.0.0";
-        iface.netmask.v6 = "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff";
-        iface.network.v4 = "127.0.0.0";
-        iface.network.v6 = "::1";
+        iface.ipv4_bindings.emplace_back(binding { "127.0.0.1", "255.0.0.0", "127.0.0.0"});
+        iface.ipv4_bindings.emplace_back(binding { "123.123.123.123", "255.255.255.0", "123.123.123.0"});
+        iface.ipv6_bindings.emplace_back(binding { "fe80::1", "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", "::1"});
+        iface.ipv6_bindings.emplace_back(binding { "fe80::2", "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", "::2"});
         iface.macaddress = "00:00:00:00:00:00";
         iface.mtu = 12345;
         result.interfaces.emplace_back(move(iface));
@@ -350,12 +394,17 @@ protected:
         // The xen fact only resolves if virtualization is xen_privileged.
         return vm::xen_privileged;
     }
+
+    virtual string get_cloud_provider(collection& facts) override
+    {
+        return "azure";
+    }
 };
 
 struct xen_resolver : resolvers::xen_resolver
 {
 protected:
-    virtual string xen_command()
+    virtual string xen_command() override
     {
         return "";
     }
@@ -371,7 +420,7 @@ protected:
 struct zfs_resolver : resolvers::zfs_resolver
 {
 protected:
-    virtual string zfs_command()
+    virtual string zfs_command() override
     {
         return "";
     }
@@ -380,7 +429,7 @@ protected:
     {
         data result;
         result.version = 1;
-        result.features = { "1", "2", "3" };
+        result.versions = { "1", "2", "3" };
         return result;
     }
 };
@@ -408,7 +457,7 @@ protected:
 struct zpool_resolver : resolvers::zpool_resolver
 {
 protected:
-    virtual string zpool_command()
+    virtual string zpool_command() override
     {
         return "";
     }
@@ -417,7 +466,8 @@ protected:
     {
         data result;
         result.version = 1;
-        result.features = { "1", "2", "3" };
+        result.versions = { "1", "2", "3" };
+        result.feature_flags = { "async_destroy", "lz4_compress", "enabled_txg" };
         return result;
     }
 };
@@ -426,17 +476,21 @@ void add_all_facts(collection& facts)
 {
     facts.add("env_windows_installdir", make_value<string_value>("C:\\Program Files\\Some\\Path"));
     facts.add("facterversion", make_value<string_value>("version"));
+    facts.add("aio_agent_version", make_value<string_value>(""));
     facts.add(make_shared<augeas_resolver>());
     facts.add(make_shared<disk_resolver>());
     facts.add(make_shared<dmi_resolver>());
     facts.add(make_shared<filesystem_resolver>());
+    facts.add(make_shared<fips_resolver>());
     // TODO: refactor the EC2 resolver to use the "collect_data" pattern
-    facts.add(fact::ec2_metadata, make_value<map_value>());
-    facts.add(fact::ec2_userdata, make_value<string_value>("user data"));
+    facts.add(make_shared<ec2_resolver>());
     // TODO: refactor the GCE resolver to use the "collect_data" pattern
+    facts.add(make_shared<hypervisors_resolver_test>());
     facts.add(fact::gce, make_value<map_value>());
     facts.add(make_shared<identity_resolver>());
     facts.add(make_shared<kernel_resolver>());
+    facts.add(fact::ldom, make_value<map_value>());
+    facts.add("ldom_domainname", make_value<string_value>("somedomain", true));
     facts.add(make_shared<load_average_resolver>());
     facts.add(make_shared<memory_resolver>());
     facts.add(make_shared<networking_resolver>());
@@ -462,15 +516,16 @@ void validate_attributes(YAML::Node const& node)
     for (auto const& attribute : node) {
         auto attribute_name = attribute.first.as<string>();
         CAPTURE(attribute_name);
-        REQUIRE_THAT(attribute_name, AnyOf(
-            Catch::Equals("pattern"),
-            Catch::Equals("type")).add(
-            Catch::Equals("hidden")).add(
-            Catch::Equals("description")).add(
-            Catch::Equals("resolution")).add(
-            Catch::Equals("caveats")).add(
-            Catch::Equals("elements")).add(
-            Catch::Equals("validate"))
+        REQUIRE_THAT(attribute_name,
+            Catch::Equals("pattern") ||
+            Catch::Equals("type") ||
+            Catch::Equals("hidden") ||
+            Catch::Equals("description") ||
+            Catch::Equals("resolution") ||
+            Catch::Equals("caveats") ||
+            Catch::Equals("elements") ||
+            Catch::Equals("validate") ||
+            Catch::Equals("blockgroup")
         );
     }
 
@@ -487,16 +542,16 @@ void validate_attributes(YAML::Node const& node)
     REQUIRE(type_attribute);
     REQUIRE(type_attribute.IsScalar());
     auto type = type_attribute.as<string>();
-    REQUIRE_THAT(type, AnyOf(
-        Catch::Equals("integer"),
-        Catch::Equals("double")).add(
-        Catch::Equals("string")).add(
-        Catch::Equals("boolean")).add(
-        Catch::Equals("array")).add(
-        Catch::Equals("map")).add(
-        Catch::Equals("ip")).add(
-        Catch::Equals("ip6")).add(
-        Catch::Equals("mac"))
+    REQUIRE_THAT(type,
+        Catch::Equals("integer") ||
+        Catch::Equals("double") ||
+        Catch::Equals("string") ||
+        Catch::Equals("boolean") ||
+        Catch::Equals("array") ||
+        Catch::Equals("map") ||
+        Catch::Equals("ip") ||
+        Catch::Equals("ip6") ||
+        Catch::Equals("mac")
     );
 
     // Check map types
@@ -555,6 +610,13 @@ void validate_attributes(YAML::Node const& node)
         REQUIRE_FALSE(caveats.empty());
     }
 
+    auto blockgroup_attribute = node["blockgroup"];
+    if (blockgroup_attribute) {
+        REQUIRE(blockgroup_attribute.IsScalar());
+        auto blockgroup = blockgroup_attribute.as<string>();
+        REQUIRE_FALSE(blockgroup.empty());
+    }
+
     // Recurse on elements
     if (elements) {
         for (auto const& element : elements) {
@@ -655,6 +717,7 @@ void validate_fact(YAML::Node const& node, value const* fact_value, bool require
     }
 }
 
+// TestRail C62454
 SCENARIO("validating schema") {
     boost::nowide::ifstream stream(LIBFACTER_TESTS_DIRECTORY "/../schema/facter.yaml");
 
@@ -706,5 +769,24 @@ SCENARIO("validating schema") {
                 return true;
             });
         }
+        THEN("blocked facts are not resolved") {
+            set<string> blocked_facts;
+            set<string> blockgroups;
+            for (auto const& fact : schema) {
+                auto fact_name = fact.first.as<string>();
+                auto blockgroup = fact.second["blockgroup"];
+                if (blockgroup) {
+                    blockgroups.insert(blockgroup.as<string>());
+                    blocked_facts.insert(fact_name);
+                }
+            }
+            collection_fixture with_blocked(blockgroups);
+            add_all_facts(with_blocked);
+            with_blocked.resolve_facts();
+            for (auto const& fact : blocked_facts) {
+                CAPTURE(fact);
+                REQUIRE_FALSE(with_blocked.get<map_value>(fact));
+            }
+        }
     }
 }
